@@ -130,7 +130,10 @@ class CreateWorkerOptions(MakerBase):
          "(None for unlimited)"],
         ["allow-shutdown", "a", None,
          "Allows the worker to initiate a graceful shutdown. One of "
-         "'signal' or 'file'"]
+         "'signal' or 'file'"],
+        ["protocol", None, "pb", "Protocol to be used when creating master-worker connection"],
+        ["proxy-connection-string", None, None,
+         "Address of HTTP proxy to tunnel through"]
     ]
 
     longdesc = textwrap.dedent("""
